#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted

from .context import amplpy
import unittest


class TestBase(unittest.TestCase):
    def setUp(self):
        self.ampl = amplpy.AMPL()

    def tearDown(self):
        self.ampl.close()


if __name__ == '__main__':
    unittest.main()



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.12
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info >= (2, 7, 0):
    def swig_import_helper():
        import importlib
        pkg = __name__.rpartition('.')[0]
        mname = '.'.join((pkg, '_amplpython')).lstrip('.')
        try:
            return importlib.import_module(mname)
        except ImportError:
            return importlib.import_module('_amplpython')
    _amplpython = swig_import_helper()
    del swig_import_helper
elif _swig_python_version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_amplpython', [dirname(__file__)])
        except ImportError:
            import _amplpython
            return _amplpython
        try:
            _mod = imp.load_module('_amplpython', fp, pathname, description)
        finally:
            if fp is not None:
                fp.close()
        return _mod
    _amplpython = swig_import_helper()
    del swig_import_helper
else:
    import _amplpython
# pull in all the attributes from _amplpython
if __name__.rpartition('.')[0] != '':
    if _swig_python_version_info >= (2, 7, 0):
        try:
            from ._amplpython import *
        except ImportError:
            from _amplpython import *
    else:
        from _amplpython import *
else:
    from _amplpython import *
del _swig_python_version_info

try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr(self, class_type, name):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    raise AttributeError("'%s' object has no attribute '%s'" % (class_type.__name__, name))


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except __builtin__.Exception:
    class _object:
        pass
    _newclass = 0

try:
    import weakref
    weakref_proxy = weakref.proxy
except __builtin__.Exception:
    weakref_proxy = lambda x: x










































































# This file is compatible with both classic and new-style classes.





# -*- coding: utf-8 -*-
from __future__ import absolute_import

import os
import platform

if platform.system() == 'Windows':
    lib32 = os.path.join(os.path.dirname(__file__), 'lib32')
    lib64 = os.path.join(os.path.dirname(__file__), 'lib64')
    os.environ['PATH'] += os.pathsep + lib32
    os.environ['PATH'] += os.pathsep + lib64


from .amplpython import *
from .amplpython import _READTABLE, _WRITETABLE



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring

from .base import BaseClass
from .utils import Utils, Tuple
from .dataframe import DataFrame
from .iterators import InstanceIterator
try:
    import pandas as pd
except ImportError:
    pd = None


class Entity(BaseClass):
    """
    An AMPL entity such as a parameter or a variable.

    An entity can either represent a single instance of an AMPL algebraic
    entity or, if the corresponding declaration has an indexing expression, a
    mapping from keys to instances. In the derived classes, it has methods to
    access instance-level properties which can be used in case the represented
    entity is scalar.

    To gain access to all the values in an entity (for all instances and all
    suffixes for that entities), use the function
    :func:`~amplpy.Entity.getValues`.

    The algebraic entities which currenty have an equivalent class in the API
    are:

    - Variables (see :class:`~amplpy.Variable`)
    - Constraints (see :class:`~amplpy.Constraint`)
    - Objectives (see :class:`~amplpy.Objective`)
    - Sets (see :class:`~amplpy.Set`)
    - Parameters (see :class:`~amplpy.Parameter`)
    """

    def __init__(self, _impl, wrapFunction=None):
        BaseClass.__init__(self, _impl)
        self.wrapFunction = wrapFunction

    def __iter__(self):
        assert self.wrapFunction is not None
        return InstanceIterator(self._impl, self.wrapFunction)

    def __getitem__(self, index):
        if not isinstance(index, (tuple, list)):
            index = [index]
        return self.get(*index)

    def __setitem__(self, index, value):
        item = self.__getitem__(index)
        if isinstance(value, (float, int, basestring)):
            item.set(value)
        else:
            item.setValues(value)

    def get(self, *index):
        """
        Get the instance with the specified index.

        Returns:
            The corresponding instance.
        """
        assert self.wrapFunction is not None
        if len(index) == 1 and isinstance(index[0], (tuple, list)):
            index = index[0]
        if len(index) == 0:
            return self.wrapFunction(self._impl.get())
        else:
            return self.wrapFunction(self._impl.get(Tuple(index)._impl))

    def find(self, *index):
        """
        Searches the current entity for an instance with the specified index.

        Returns:
            The wanted instance if found, otherwise it returns `None`.
        """
        assert self.wrapFunction is not None
        if len(index) == 1 and isinstance(index[0], (tuple, list)):
            index = index[0]
        it = self._impl.find(Tuple(index)._impl)
        if it == self._impl.end():
            return None
        else:
            return self.wrapFunction(it)

    def instances(self):
        """
        Get all the instances in this entity..
        """
        return InstanceIterator(self._impl, self.wrapFunction)

    def name(self):
        """
        Get the name of this entity.
        """
        return self._impl.name()

    def indexarity(self):
        """
        Get the indexarity of this entity (sum of the dimensions of the
        indexing sets).
        This value indicates the arity of the tuple to be passed to the method
        :func:`~amplpy.Entity.get` in order to access an instance of this
        entity. See the following AMPL examples:

        .. code-block:: ampl

            var x;               # indexarity = 0
            var y {1..2};        # indexarity = 1
            var z {1..2,3..4};   # indexarity = 2
            var zz {{(1, 2)}};   # indexarity = 2

        Returns:
            The sum of the dimensions of the indexing sets or 0 if the
            entity is not indexed.
        """
        return self._impl.indexarity()

    def isScalar(self):
        """
        Check whether this entity is scalar. Equivalent to testing whether
        :func:`~amplpy.Entity.indexarity` is equal to zero.

        Returns:
            True if the entity is scalar (not indexed over any set).
        """
        return self._impl.isScalar()

    def numInstances(self):
        """
        Get the number of instances in this entity.
        """
        return self._impl.numInstances()

    def getIndexingSets(self):
        """
        Get the AMPL string representation of the sets on which this entity is
        indexed. The obtained vector can be modified without any effect to the
        entity.

        Returns:
            The string representation of the indexing sets for this entity or
            an empty array if the entity is scalar.
        """
        return Utils.castStringArray(self._impl.getIndexingSets())

    def getValues(self, suffixes=None):
        """
        If a list of suffixes is provided, get the specified suffixes value for
        all instances. Otherwise, get all the principal values of this entity.
        The specific returned value depends on the type of entity (see list
        below). For:

        - Variables and Objectives it returns the suffix ``val``.
        - Parameters it returns their values.
        - Constraints it returns the suffix ``dual``.
        - Sets it returns all the members of the set. Note that it does not
          apply to indexed sets. See :func:`~amplpy.Set.getValues`.

        Retruns:
            A :class:`~amplpy.DataFrame` containing the values for all
            instances.
        """
        if suffixes is None:
            return DataFrame._fromDataFrameRef(self._impl.getValues())
        else:
            suffixes = list(map(str, suffixes))
            return DataFrame._fromDataFrameRef(
                self._impl.getValuesLst(suffixes, len(suffixes))
            )

    def setValues(self, data):
        """
        Set the values of this entiy to the correponding values of a
        DataFrame indexed over the same sets (or a subset).
        This function assigns the values in the first data column of
        the passed dataframe to the entity the function is called from.
        In particular, the statement:

        .. code-block:: python

            x.setValues(y.getValues())

        is semantically equivalent to the AMPL statement:

        .. code-block:: ampl

            let {s in S} x[s] := y[s];

        Args:
            data: The data to set the entity to.
        """
        if isinstance(data, DataFrame):
            self._impl.setValuesDf(data._impl)
        else:
            if pd is not None and isinstance(data, (pd.DataFrame, pd.Series)):
                df = DataFrame.fromPandas(data)
                self._impl.setValuesDf(df._impl)
                return
            raise TypeError



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted

from . import TestBase
import unittest
import amplpy


class TestIterators(TestBase.TestBase):
    """Test Iterators."""

    def testSetIterators(self):
        ampl = self.ampl
        ampl.eval('set x;')
        self.assertEqual(
            len(ampl.getSets()),
            ampl.getSets().size()
        )
        self.assertEqual(
            ampl.getSets().size(),
            len(list(ampl.getSets()))
        )
        self.assertEqual(ampl.getSets().size(), 1)
        ampl.eval('reset;')
        self.assertEqual(ampl.getSets().size(), 0)
        ampl.eval('set x;')
        self.assertEqual(ampl.getSets().size(), 1)
        ampl.eval('set y{1..2};')
        self.assertEqual(ampl.getSets().size(), 2)
        ampl.eval('set z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.getSets().size(), 3)
        ampl.eval('set xx{1..2,4..5};')
        self.assertEqual(ampl.getSets().size(), 4)
        ampl.eval('set yy{1..2,4..5,7..8};')
        self.assertEqual(ampl.getSets().size(), 5)
        ampl.eval('set zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.getSets().size(), 6)
        self.assertEqual(
            ampl.getSet('x').numInstances(),
            len(dict(ampl.getSet('x')))
        )
        self.assertEqual(ampl.getSet('x').numInstances(), 1)
        self.assertEqual(ampl.getSet('y').numInstances(), 2)
        self.assertEqual(ampl.getSet('z').numInstances(), 4)
        self.assertEqual(ampl.getSet('xx').numInstances(), 4)
        self.assertEqual(ampl.getSet('yy').numInstances(), 8)
        self.assertEqual(ampl.getSet('zz').numInstances(), 8)
        self.assertEqual(
            max(s.numInstances() for name, s in ampl.getSets()), 8
        )
        self.assertEqual(
            ampl.getSet('x').name(),
            ampl.getSets()['x'].name()
        )

    def testParameterIterators(self):
        ampl = self.ampl
        ampl.eval('param x;')
        self.assertEqual(
            len(ampl.getParameters()),
            ampl.getParameters().size()
        )
        self.assertEqual(
            ampl.getParameters().size(),
            len(list(ampl.getParameters()))
        )
        self.assertEqual(ampl.getParameters().size(), 1)
        ampl.eval('reset;')
        self.assertEqual(ampl.getParameters().size(), 0)
        ampl.eval('param x;')
        self.assertEqual(ampl.getParameters().size(), 1)
        ampl.eval('param y{1..2};')
        self.assertEqual(ampl.getParameters().size(), 2)
        ampl.eval('param z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.getParameters().size(), 3)
        ampl.eval('param xx{1..2,4..5};')
        self.assertEqual(ampl.getParameters().size(), 4)
        ampl.eval('param yy{1..2,4..5,7..8};')
        self.assertEqual(ampl.getParameters().size(), 5)
        ampl.eval('param zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.getParameters().size(), 6)
        self.assertEqual(
            ampl.getParameter('x').numInstances(),
            len(dict(ampl.getParameter('x')))
        )
        self.assertEqual(ampl.getParameter('x').numInstances(), 1)
        self.assertEqual(ampl.getParameter('y').numInstances(), 2)
        self.assertEqual(ampl.getParameter('z').numInstances(), 4)
        self.assertEqual(ampl.getParameter('xx').numInstances(), 4)
        self.assertEqual(ampl.getParameter('yy').numInstances(), 8)
        self.assertEqual(ampl.getParameter('zz').numInstances(), 8)
        self.assertEqual(
            max(p.numInstances() for name, p in ampl.getParameters()), 8
        )
        self.assertEqual(
            ampl.getParameter('x').name(),
            ampl.getParameters()['x'].name()
        )

    def testVariableIterators(self):
        ampl = self.ampl
        ampl.eval('var x;')
        self.assertEqual(
            len(ampl.getVariables()),
            ampl.getVariables().size()
        )
        self.assertEqual(
            ampl.getVariables().size(),
            len(list(ampl.getVariables()))
        )
        self.assertEqual(ampl.getVariables().size(), 1)
        ampl.eval('reset;')
        self.assertEqual(ampl.getVariables().size(), 0)
        ampl.eval('var x;')
        self.assertEqual(ampl.getVariables().size(), 1)
        ampl.eval('var y{1..2};')
        self.assertEqual(ampl.getVariables().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.getVariables().size(), 3)
        ampl.eval('var xx{1..2,4..5};')
        self.assertEqual(ampl.getVariables().size(), 4)
        ampl.eval('var yy{1..2,4..5,7..8};')
        self.assertEqual(ampl.getVariables().size(), 5)
        ampl.eval('var zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.getVariables().size(), 6)
        self.assertEqual(
            ampl.getVariable('x').numInstances(),
            len(dict(ampl.getVariable('x')))
        )
        self.assertEqual(ampl.getVariable('x').numInstances(), 1)
        self.assertEqual(ampl.getVariable('y').numInstances(), 2)
        self.assertEqual(ampl.getVariable('z').numInstances(), 4)
        self.assertEqual(ampl.getVariable('xx').numInstances(), 4)
        self.assertEqual(ampl.getVariable('yy').numInstances(), 8)
        self.assertEqual(ampl.getVariable('zz').numInstances(), 8)
        self.assertEqual(
            max(var.numInstances() for name, var in ampl.getVariables()), 8
        )
        self.assertEqual(
            ampl.getVariable('x').name(),
            ampl.getVariables()['x'].name()
        )

    def testConstraintIterators(self):
        ampl = self.ampl
        ampl.eval('var x;')
        ampl.eval('s.t. c_x: x = 0;')
        self.assertEqual(
            ampl.getConstraints().size(),
            len(ampl.getConstraints())
        )
        self.assertEqual(
            ampl.getConstraints().size(),
            len(list(ampl.getConstraints()))
        )
        self.assertEqual(ampl.getConstraints().size(), 1)
        ampl.eval('reset;')
        self.assertEqual(ampl.getConstraints().size(), 0)
        ampl.eval('var x;')
        ampl.eval('s.t. c_x: x = 0;')
        self.assertEqual(ampl.getConstraints().size(), 1)
        ampl.eval('var y{1..2};')
        ampl.eval('s.t. c_y{i in 1..2}: y[i] = i;')
        self.assertEqual(ampl.getConstraints().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        ampl.eval('s.t. c_z{(i, j) in {1..2, {"a", "b"}}}: z[i,j] = i;')
        self.assertEqual(ampl.getConstraints().size(), 3)
        ampl.eval('var xx{1..2,4..5};')
        ampl.eval('s.t. c_xx{(i, j) in {1..2,4..5}}: xx[i,j] = i+j;')
        self.assertEqual(ampl.getConstraints().size(), 4)
        self.assertEqual(
            ampl.getConstraint('c_x').numInstances(),
            len(dict(ampl.getConstraint('c_x')))
        )
        self.assertEqual(ampl.getConstraint('c_x').numInstances(), 1)
        self.assertEqual(ampl.getConstraint('c_y').numInstances(), 2)
        self.assertEqual(ampl.getConstraint('c_z').numInstances(), 4)
        self.assertEqual(ampl.getConstraint('c_xx').numInstances(), 4)
        self.assertEqual(
            max(con.numInstances() for name, con in ampl.getConstraints()), 4
        )
        self.assertEqual(
            ampl.getConstraint('c_x').name(),
            ampl.getConstraints()['c_x'].name()
        )

    def testObjectiveIterators(self):
        ampl = self.ampl
        ampl.eval('var x;')
        ampl.eval('maximize c_x: x;')
        self.assertEqual(
            ampl.getObjectives().size(),
            len(ampl.getObjectives())
        )
        self.assertEqual(
            ampl.getObjectives().size(),
            len(list(ampl.getObjectives()))
        )
        self.assertEqual(ampl.getObjectives().size(), 1)
        ampl.eval('reset;')
        self.assertEqual(ampl.getObjectives().size(), 0)
        ampl.eval('var x;')
        ampl.eval('maximize c_x: x;')
        self.assertEqual(ampl.getObjectives().size(), 1)
        ampl.eval('var y{1..2};')
        ampl.eval('maximize c_y{i in 1..2}: y[i];')
        self.assertEqual(ampl.getObjectives().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        ampl.eval('maximize c_z{(i, j) in {1..2, {"a", "b"}}}: z[i,j];')
        self.assertEqual(ampl.getObjectives().size(), 3)
        ampl.eval('var xx{1..2,4..5};')
        ampl.eval('maximize c_xx{(i, j) in {1..2,4..5}}: xx[i,j];')
        self.assertEqual(ampl.getObjectives().size(), 4)
        self.assertEqual(
            ampl.getObjective('c_x').numInstances(),
            len(dict(ampl.getObjective('c_x')))
        )
        self.assertEqual(ampl.getObjective('c_x').numInstances(), 1)
        self.assertEqual(ampl.getObjective('c_y').numInstances(), 2)
        self.assertEqual(ampl.getObjective('c_z').numInstances(), 4)
        self.assertEqual(ampl.getObjective('c_xx').numInstances(), 4)
        self.assertEqual(
            max(obj.numInstances() for name, obj in ampl.getObjectives()), 4
        )
        self.assertEqual(
            ampl.getObjective('c_x').name(),
            ampl.getObjectives()['c_x'].name()
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring
from numbers import Real

from .base import BaseClass
from .utils import Utils, Tuple
from .iterators import RowIterator, ColIterator
from . import amplpython
try:
    import pandas as pd
except ImportError:
    pd = None
try:
    import numpy as np
except ImportError:
    np = None


class Row(BaseClass):
    """
    Represents a row in a :class:`~amplpy.DataFrame`.
    """

    def __init__(self, _impl):
        self._impl = _impl

    def __iter__(self):
        return RowIterator(self._impl)

    def __getitem__(self, key):
        return Utils.castVariantRef(self._impl.getIndex(key))

    def toString(self):
        return str(list(self))


class Column(BaseClass):
    """
    Represents a column in a :class:`~amplpy.DataFrame`.
    """

    def __init__(self, _impl):
        self._impl = _impl

    def __iter__(self):
        return ColIterator(self._impl)

    def toString(self):
        return str(list(self))


class DataFrame(BaseClass):
    """
    A DataFrame object, used to communicate data to and from the AMPL entities.

    An object of this class can be used to do the following tasks:
    - Assign values to AMPL entities (once the DataFrame is populated, use
    :func:`~amplpy.AMPL.setData` to assign its values to the modelling entities
    in its columns)
    - Get values from AMPL, decoupling the values from the AMPL entities they
    originate via :func:`~amplpy.Entity.getValues`.

    A DataFrame object can be created in various ways.

    - Create a skeleton by specifiying manually the indexing columns and the
      column headers.
    - Get values from AMPL, decoupling the values from the AMPL entities they
      originate from (via :func:`~amplpy.Entity.getValues`).

    Populating a DataFrame object can be done adding row by row to a
    pre-existing skeleton via :func:`~amplpy.DataFrame.addRow`, setting whole
    columns of a pre-existing skeleton via :func:`~amplpy.DataFrame.setColumn`
    or adding columns (including indexing columns) via
    :func:`~amplpy.DataFrame.addColumn`.

    Modifying a DataFrame object can be done via
    :func:`~amplpy.DataFrame.setColumn` or, item by item, via
    :func:`~amplpy.DataFrame.setValue`.

    Accessing data in a DataFrame can be done row by row using
    :func:`~amplpy.DataFrame.getRow` or by column via
    :func:`~amplpy.DataFrame.getColumn`.
    """

    def __init__(self, index, columns=tuple(), **kwargs):
        """
        Create a new DataFrame with specifed index and column headers.

        Args:
            index: Index column;

            columns: Column headers.
        """
        if index is not None:
            if isinstance(index, basestring):
                index = (index,)
            if isinstance(columns, basestring):
                columns = (columns,)
            index_names = [
                col[0] if isinstance(col, tuple) else col
                for col in index
            ]
            column_names = [
                col[0] if isinstance(col, tuple) else col
                for col in columns
            ]
            self._impl = amplpython.DataFrame.factory(
                len(index_names),
                list(index_names) + list(column_names),
                len(index_names) + len(column_names)
            )
            for col in index:
                if isinstance(col, tuple):
                    self.setColumn(col[0], col[1])
            for col in columns:
                if isinstance(col, tuple):
                    self.setColumn(col[0], col[1])
        else:
            self._impl = kwargs.get('_impl', None)

    def __iter__(self):
        # FIXME: C++ iterators for dataframes not working with SWIG.
        return (self.getRowByIndex(i) for i in range(self.getNumRows()))

    def getNumCols(self):
        """
        Get the total number of columns in this dataframe (indexarity + number
        of values).

        Returns:
            The number of columns.
        """
        return self._impl.getNumCols()

    def getNumRows(self):
        """
        Get the number of data rows in this dataframe.

        Returns:
            The number of rows.
        """
        return self._impl.getNumRows()

    def getNumIndices(self):
        """
        Get the number of indices (the indexarity) of this dataframe.

        Returns:
            The number of indices needed to access one row of this dataframe.
        """
        return self._impl.getNumIndices()

    def addRow(self, *value):
        """
         Add a row to the DataFrame. The size of the tuple must be equal to the
         total number of columns in the dataframe.

         Args:
            value: A single argument with a tuple containing all the values
            for the row to be added, or multiple arguments with the values for
            each column.
        """
        if len(value) == 1 and isinstance(value[0], (tuple, list)):
            value = value[0]
        assert len(value) == self.getNumCols()
        self._impl.addRow(Tuple(value)._impl)

    def addColumn(self, header, values=[]):
        """
        Add a new column with the corresponding header and values to the
        dataframe.

        Args:
            header: The name of the new column.

            values: A list of size :func:`~amplpy.DataFrame.getNumRows` with
            all the values of the new column.
        """
        if len(values) == 0:
            self._impl.addColumn(header)
        else:
            assert len(values) == self.getNumRows()
            if any(isinstance(value, basestring) for value in values):
                values = list(map(str, values))
                self._impl.addColumnStr(header, values)
            elif all(isinstance(value, Real) for value in values):
                values = list(map(float, values))
                self._impl.addColumnDbl(header, values)
            else:
                raise NotImplementedError

    def getColumn(self, header):
        """
        Get the specified column as a view object.

        Args:
            header: The header of the column.
        """
        return Column(self._impl.getColumn(header))

    def setColumn(self, header, values):
        """
        Set the values of a column.

        Args:
            header: The header of the column to be set.

            values: The values to set.
        """
        if any(isinstance(value, basestring) for value in values):
            values = list(map(str, values))
            self._impl.setColumnStr(header, values, len(values))
        elif all(isinstance(value, Real) for value in values):
            values = list(map(float, values))
            self._impl.setColumnDbl(header, values, len(values))
        else:
            print(values)
            raise NotImplementedError

    def getRow(self, key):
        """
        Get a row by value of the indexing columns. If the index is not
        specified, gets the only row of a dataframe with no indexing columns.

        Args:
            key: Tuple representing the index of the desired row.

        Returns:
            The row.
        """
        return Row(self._impl.getRow(Tuple(key)._impl))

    def getRowByIndex(self, index):
        """
        Get row by numeric index.

        Args:
            index: Zero-based index of the row to get.

        Returns:
            The corresponding row.
        """
        assert isinstance(index, int)
        return Row(self._impl.getRowByIndex(index))

    def getHeaders(self):
        """
         Get the headers of this DataFrame.

         Returns:
            The headers of this DataFrame.
        """
        headers = self._impl.getHeaders()
        return tuple(
            headers.getIndex(i) for i in range(self._impl.getNumCols())
        )

    def setValues(self, values):
        """
        Set the values of a DataFrame from a dictionary.

        Args:
            values: Dictionary with the values to set.
        """
        ncols = self.getNumCols()
        nindices = self.getNumIndices()
        for key, value in values.items():
            key = Utils.convToList(key)
            assert len(key) == nindices
            value = Utils.convToList(value)
            assert len(value) == ncols-nindices
            self.addRow(key + value)

    def toDict(self):
        """
        Return a dictionary with the DataFrame data.
        """
        d = {}
        nindices = self.getNumIndices()
        for i in range(self.getNumRows()):
            row = list(self.getRowByIndex(i))
            if nindices > 1:
                key = tuple(row[:nindices])
            elif nindices == 1:
                key = row[0]
            else:
                key = None
            if len(row) - nindices == 0:
                d[key] = None
            elif len(row) - nindices == 1:
                d[key] = row[nindices]
            else:
                d[key] = tuple(row[nindices:])
        return d

    def toList(self):
        """
        Return a list with the DataFrame data.
        """
        if self.getNumCols() > 1:
            return [
                tuple(self.getRowByIndex(i))
                for i in range(self.getNumRows())
            ]
        else:
            return [
                self.getRowByIndex(i)[0]
                for i in range(self.getNumRows())
            ]

    def toPandas(self):
        """
        Return a pandas DataFrame with the DataFrame data.
        """
        assert pd is not None
        nindices = self.getNumIndices()
        headers = self.getHeaders()
        columns = {
            header: list(self.getColumn(header))
            for header in headers[nindices:]
        }
        index = zip(*[
            list(self.getColumn(header))
            for header in headers[:nindices]
        ])
        index = [key if len(key) > 1 else key[0] for key in index]
        if index == []:
            return pd.DataFrame(columns, index=None)
        else:
            return pd.DataFrame(columns, index=index)

    @classmethod
    def fromPandas(cls, df):
        """
        Create a :class:`~amplpy.DataFrame` from a pandas DataFrame.
        """
        assert pd is not None
        if isinstance(df, pd.Series):
            df = pd.DataFrame(df)
        else:
            assert isinstance(df, pd.DataFrame)
        keys = [
            key if isinstance(key, tuple) else (key,)
            for key in df.index.tolist()
        ]
        index = [
            ('index{}'.format(i), cindex)
            for i, cindex in enumerate(zip(*keys))
        ]
        columns = [
            (str(cname), df[cname].tolist())
            for cname in df.columns.tolist()
        ]
        return cls(index=index, columns=columns)

    @classmethod
    def fromNumpy(cls, data):
        """
        Create a :class:`~amplpy.DataFrame` from a numpy array or matrix.
        """
        assert np is not None
        if isinstance(data, np.ndarray):
            index = []
            if len(data.shape) == 1:
                columns = [('value', data.tolist())]
            elif len(data.shape) == 2:
                columns = [
                    ('c{}'.format(i), col)
                    for i, col in enumerate(zip(*data.tolist()))
                ]
            else:
                raise TypeError
        else:
            raise TypeError
        return cls(index=index, columns=columns)

    @classmethod
    def _fromDataFrameRef(cls, dfRef):
        return cls(None, None, _impl=dfRef)



# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, division
from builtins import map, range, object, zip, sorted
from past.builtins import basestring
from numbers import Real

from .entity import Entity
from .utils import Utils, Tuple
from .dataframe import DataFrame
try:
    import numpy as np
except ImportError:
    np = None


class Parameter(Entity):
    """
    Represents an AMPL parameter. The values can be float or string (in case of
    symbolic parameters).

    Data can be assigned to the set using the methods
    :func:`~amplpy.Parameter.set` and :func:`~amplpy.Parameter.setValues` or
    using :func:`~amplpy.AMPL.setData` and an object of class
    :class:`~amplpy.DataFrame`.
    """

    def __init__(self, _impl):
        Entity.__init__(
            self,
            _impl,
            lambda it: Utils.castVariantRef(it)
        )

    def __setitem__(self, index, value):
        self.set(index, value)

    def isSymbolic(self):
        """
        Returns True if the parameter is declared as symbolic (can store both
        numerical and string values).
        """
        return self._impl.isSymbolic()

    def hasDefault(self):
        """
        Check if the parameter has a default initial value. In case of the
        following AMPL code:

        .. code-block:: ampl

            param a;
            param b default a;

        the function will return true for parameter ``b``.

        Returns:
            True if the parameter has a default initial value. Please note
            that if the parameter has a default expression which refers to
            another parameter which value is not defined, this will return
            True.
        """
        return self._impl.hasDefault()

    def value(self):
        """
        Get the value of this parameter. Valid only for non-indexed parameters.
        """
        return self.get()

    def set(self, *args):
        """
        Set the value of a single instance of this parameter.

        Args:
            args: value if the parameter is scalar, index and value
            otherwise.

        Raises:
            RuntimeError: If the entity has been deleted in the underlying
            AMPL.

            TypeError: If the parameter is not scalar and the index is not
            provided.
        """
        assert len(args) in (1, 2)
        if len(args) == 1:
            value = args[0]
            self._impl.set(value)
        else:
            index, value = args
            if isinstance(value, Real):
                self._impl.setTplDbl(Tuple(index)._impl, value)
            elif isinstance(value, basestring):
                self._impl.setTplStr(Tuple(index)._impl, value)
            else:
                raise TypeError

    def setValues(self, values):
        """
        Assign the values (string or float) to the parameter instances with the
        specified indices, equivalent to the AMPL code:

        .. code-block:: ampl

            let {i in indices} par[i] := values[i];

        Args:
            values: list, dictionary or :class:`~amplpy.DataFrame` with the
            indices and the values to be set.

        Raises:
            TypeError: If called on a scalar parameter.
        """
        if isinstance(values, dict):
            indices, values = list(zip(*values.items()))
            indices = Utils.toTupleArray(indices)
            if any(isinstance(value, basestring) for value in values):
                values = list(map(str, values))
                self._impl.setValuesTaStr(indices, values, len(values))
            elif all(isinstance(value, Real) for value in values):
                values = list(map(float, values))
                self._impl.setValuesTaDbl(indices, values, len(values))
            else:
                raise TypeError
        elif isinstance(values, (list, tuple)):
            if any(isinstance(value, basestring) for value in values):
                values = list(map(str, values))
                self._impl.setValuesStr(values, len(values))
            elif all(isinstance(value, Real) for value in values):
                values = list(map(float, values))
                self._impl.setValuesDbl(values, len(values))
            else:
                raise TypeError
        else:
            if np is not None and isinstance(values, np.ndarray):
                self.setValues(DataFrame.fromNumpy(values).toList())
                return
            Entity.setValues(self, values)
